LIBRARY IEEE, STD;
use ieee.std_logic_1164.all ;
USE std.textio.all;
----------------------------------------------------------------------------
--                                                                        --
-- Copyright (c) 1999 by Papillon Research Corp.  All rights reserved.    --
--                                                                        --
-- This source file may be used and distributed without restriction       --
-- provided that this copyright statement is not removed from the file    --
-- and that any derivative work contains this copyright notice. This      --
-- code or it's derivatives may not be sold as a whole or as a part of    --
-- any commercial product without written consent from Papillon Research. --
--                                                                        --
-- Program name: papio.vhd                                                --
--							 	          --
----------------------------------------------------------------------------
--  
--  The information in this document is subject to change without
--  notice and should not be construed as a commitment by PAPILLON.
-------------------------------------------------------------------------------
-- Title         : Papillon I/O Routines
-- File          : papio.vhd
-- Description   : collection of common routines 
-------------------------------------------------------------------------------

PACKAGE papio IS

  TYPE BASE is (DEC,BIN,OCT,HEX) ;
  CONSTANT HEXCODES : string(1 to 22) := "0123456789ABCDEFabcdef" ;
  
  
  FUNCTION to_VEC(ARG: in bit_vector; SIZE: INTEGER) return std_logic_vector ;
  FUNCTION to_VEC(ARG,SIZE: in integer) return std_logic_vector ;
  FUNCTION to_INT(ARG: in std_logic_vector) return integer ;
  FUNCTION to_INT(ARG: in std_logic) return integer ;
  FUNCTION to_STR(INT: integer; B: base) return string ; 
  FUNCTION to_STR(INT: integer) return string ;
  FUNCTION to_STR(VEC: std_logic_vector) return string ;
  FUNCTION to_STR(VEC: bit_vector) return string ;
  FUNCTION to_CHAR(ARG: integer) return character ;
  FUNCTION PAD_STR(str: string; size: INTEGER) RETURN string;
  FUNCTION to_VEC(str: string) return STD_LOGIC_VECTOR;
  FUNCTION lower_case(c : character) return character;
  FUNCTION strcmp(a, b : string) RETURN BOOLEAN;

END papio;


PACKAGE BODY papio IS

  
--********************************************************************
--*                                                                  *
--* Function to convert bit_vector to std_logic_vector               *
--*                                                                  *
--********************************************************************
 

  function to_VEC(ARG: in bit_vector; SIZE: IN INTEGER) return STD_LOGIC_VECTOR is 

    variable TEMP : BIT_VECTOR(SIZE-1 DOWNTO 0);
    variable RESULT: STD_LOGIC_VECTOR(SIZE-1 downto 0);

  begin

    TEMP := ARG ;
    
    for i in 0 to SIZE-1 loop
      if (TEMP(i) = '1') then
        RESULT(i) := '1';
      else
        RESULT(i) := '0';
      end if;
    end loop;
   return RESULT ;

  end;

--********************************************************************
--*                                                                  *
--* Function to convert an integer and return it's vector equivalent *
--*                                                                  *
--********************************************************************
 

  function to_VEC(ARG,SIZE: in INTEGER) return STD_LOGIC_VECTOR is 

    variable TEMP : integer ;
    variable RESULT: STD_LOGIC_VECTOR(SIZE-1 downto 0);

  begin

    TEMP := ARG ;
    for i in 0 to SIZE-1 loop
      if (TEMP mod 2) = 1 then
        RESULT(i) := '1';
      else
        RESULT(i) := '0';
      end if;
      TEMP := TEMP/2;
    end loop;
   return RESULT ;

  end;


--********************************************************************
--*                                                                  *
--* Function to convert an vector and return it's integer equivalent *
--*                                                                  *
--********************************************************************

  function to_INT (ARG : in std_logic_vector) return INTEGER is

    variable RESULT: integer ;

  begin

    RESULT := 0 ;

    --  djm removed the following check/warning for array size too big
    --  because it is overly pessimistic.  If the vector is > 31 bits
    --  but bit 32 is not set, then the number is not too large.
--    assert ARG'length < 32               -- Assert array size limit.
--      report "Vector Length > 31. i.e. It's too big." ;

    for I in ARG'range loop             -- Calculate unsigned value.
      RESULT := RESULT * 2 ;
      if not(ARG(i) = '0' or ARG(i) = '1') then
        assert false
          report "VEC2INT Conversion Error. " &
                 "Unable to resolve 'undefined' states." ;
        return 0 ;
      end if ;
      if ARG(i) = '1' then
        RESULT := RESULT + 1 ;
      end if ;
    end loop ;
    return result ;

  end ;

--********************************************************************
--*                                                                  *
--* Function to convert std_logic bit and return it's integer equivalent *
--*                                                                  *
--********************************************************************

  function to_INT (ARG : in std_logic) return INTEGER is

    variable RESULT: integer ;

  begin

    RESULT := 0 ;

      if not(ARG = '0' or ARG = '1') THEN
--        assert false
--          report "VEC2INT Conversion Error. " &
--                 "Unable to resolve 'undefined' states." ;
        return 0 ;
      end if ;
      if ARG = '1' then
        RESULT := 1 ;
      end if ;
    return result ;

  end ;

--********************************************************************
--*                                                                  *
--* Function to convert an integer and return it's string equivalent *
--*                                                                  *
--********************************************************************

  function to_STR(INT: integer; B: base) return string is

    variable TEMP : string(1 to 10) ;
    variable NUM,RADIX : integer := 0 ;
    variable LEN,POWER : integer := 1 ;
    variable LHS : boolean := true ;

  begin

    NUM := INT ;

    case B is
      when BIN => RADIX := 2 ;                -- Depending on what
      when OCT => RADIX := 8 ;                -- base the number is
      when HEX => RADIX := 16 ;               -- to be displayed as
      when DEC => RADIX := 10 ;               -- choose a radix range
    end case ;

    while NUM >= RADIX loop                   -- Determine how many
      LEN := LEN + 1 ;                        -- characters required
      NUM := NUM / RADIX ;                    -- to represent the
    end loop ;                                -- number.
    
    for i in LEN downto 1 loop                -- Convert the number to
      TEMP(i) := to_CHAR(INT/POWER mod RADIX) ; -- a string starting
      POWER := POWER * RADIX ;                -- with the right hand
    end loop ;                                -- side.
    return TEMP(1 to LEN) ;

  end to_STR ;



  function to_STR(INT: INTEGER) return string is

  begin

    return to_STR(INT,DEC) ;

  end to_STR ;


--********************************************************************
--*                                                                  *
--*  This function returns the STRING equivalent std_ulogic_vector   *
--*                                                                  *
--********************************************************************
  
  function to_STR(VEC: std_logic_vector) return string is
    variable RESULT : string (1 to VEC'length) ;
    variable N : character ;
  begin
    for i in vec'low to VEC'high loop
      case VEC(i) is
        when 'U' => N := 'U' ;
        when 'X' => N := 'X' ;
        when '0' => N := '0' ;
        when '1' => N := '1' ;
        when 'Z' => N := 'Z' ;
        when 'L' => N := 'L' ;
        when 'H' => N := 'H' ;
        when '-' => N := '-' ;
        when 'W' => N := 'W' ;
        when others => N := '?' ; 
      end case ;
      RESULT := N&RESULT(1 to VEC'length-1) ;
    end loop ;
    return RESULT ;    
  end ;


--********************************************************************
--*                                                                  *
--*  This function returns the STRING equivalent bit_vector          *
--*                                                                  *
--********************************************************************
  
  function to_STR(VEC: bit_vector) return string is
    variable RESULT : string (1 to VEC'length) ;
    variable N : character ;
  begin
    for i in 0 to VEC'length-1 loop
      case VEC(i) is
        when '0' => N := '0' ;
        when '1' => N := '1' ;
        when others => N := '?' ; 
      end case ;
      RESULT := N & RESULT(1 to VEC'length-1) ;
    end loop ;
    return RESULT ;    
  end ;

--********************************************************************
--*                                                                  *
--*  This function returns the character of a HEX digit - NOT ASCII  *
--*                                                                  *
--********************************************************************

  function to_CHAR(ARG: integer) return character is

    variable CHAR : character ;

  begin

    CHAR := HEXCODES(ARG+1) ;     -- Offset of 1 since string starts
    return CHAR ;                 -- at 1 not 0.

  end to_CHAR ;

  
--********************************************************************
--*                                                                  *
--*  This function pads out a string to the desired length           *
--*                                                                  *
--********************************************************************

  FUNCTION PAD_STR(str: string; size: INTEGER) RETURN string IS
    CONSTANT padchar:   character := ' ';
    VARIABLE result: string (1 TO size);
    BEGIN
      IF (size <= str'length) THEN
        RETURN str;
      END IF;

      --  Transfer the existing string:
      FOR i IN 1 TO str'length LOOP
        result(i) := str(i);
      END LOOP ;

      --  Now pad out the string to length:
      FOR i IN (str'length + 1) TO size LOOP
        result(i) := padchar;
      END LOOP ;
      RETURN result;

    END PAD_STR;


--  PROCEDURE pap_write(textin: out text; stringin: IN string) IS 
--    VARIABLE l:           line;
--    begin
--      write(l, stringin);
--      writeline(textin, l);
--    END pap_write;

--****************************************************************************
--*                                                                          *
--*  This function returns the bit_string equivalent of a HEX STRING         *
--*                                                                          *
--****************************************************************************
  
  function to_VEC(str: string) return STD_LOGIC_VECTOR is
    variable result : STD_LOGIC_VECTOR(4 TO (STR'length*4)+3);
    variable return_result : STD_LOGIC_VECTOR((STR'length*4)-1 DOWNTO 0)  := (OTHERS => '0');
  begin
    for i in 1 to str'length loop
      case str(i) is
        when '0' => result(i*4 TO (i*4)+3) := "0000" ;
        when '1' => result(i*4 TO (i*4)+3) := "0001" ;
        when '2' => result(i*4 TO (i*4)+3) := "0010" ;
        when '3' => result(i*4 TO (i*4)+3) := "0011" ;
        when '4' => result(i*4 TO (i*4)+3) := "0100" ;
        when '5' => result(i*4 TO (i*4)+3) := "0101" ;
        when '6' => result(i*4 TO (i*4)+3) := "0110" ;
        when '7' => result(i*4 TO (i*4)+3) := "0111" ;
        when '8' => result(i*4 TO (i*4)+3) := "1000" ;
        when '9' => result(i*4 TO (i*4)+3) := "1001" ;
        when 'A' => result(i*4 TO (i*4)+3) := "1010" ;
        when 'B' => result(i*4 TO (i*4)+3) := "1011" ;
        when 'C' => result(i*4 TO (i*4)+3) := "1100" ;
        when 'D' => result(i*4 TO (i*4)+3) := "1101" ;
        when 'E' => result(i*4 TO (i*4)+3) := "1110" ;
        when 'F' => result(i*4 TO (i*4)+3) := "1111" ;

        when others =>
	    ASSERT false
		REPORT "INVALID HEX STRING CHARACTER FOUND IN CONVERSION!  SETTING TO ZEROS"
		SEVERITY ERROR;
	    RETURN RETURN_result;
      end case ;
    end loop ;
    return_result := result;
    return return_result ;    
  end ;

    -- Convert characters to lower case
    FUNCTION lower_case(c : character) RETURN character
    IS
    BEGIN
	IF c >= 'A' AND c <= 'Z' THEN
	    RETURN character'val(character'pos(c) + 32);
	ELSE RETURN c;
	END IF;
    END;

    -- compare two strings ignoring case
    FUNCTION strcmp(a, b : string) RETURN BOOLEAN
    IS
	ALIAS a_val : string(1 to a'length) is a;
	ALIAS b_val : string(1 to b'length) is b;
	VARIABLE a_char, b_char : character;
    BEGIN
	IF a'length /= b'length THEN
	    RETURN FALSE;
	ELSIF a = b THEN
	    RETURN TRUE;
	END IF;
	FOR i IN 1 TO a'length LOOP
	    a_char := lower_case(a_val(i));
	    b_char := lower_case(b_val(i));
	    IF a_char /= b_char THEN
		RETURN FALSE;
	    END IF;
	END LOOP;
	RETURN TRUE;
    END;


    
END papio;
