----------------------------------------------------------------------------
--                                                                        --
-- Copyright (c) 1999 by Papillon Research Corp.  All rights reserved.    --
--                                                                        --
-- This source file may be used and distributed without restriction       --
-- provided that this copyright statement is not removed from the file    --
-- and that any derivative work contains this copyright notice. This      --
-- code or it's derivatives may not be sold as a whole or as a part of    --
-- any commercial product without written consent from Papillon Research. --
--                                                                        --
-- Program name: papio.vhd                                                --
--							 	          --
----------------------------------------------------------------------------
--  
--  The information in this document is subject to change without
--  notice and should not be construed as a commitment by PAPILLON.
-------------------------------------------------------------------------------
-- Title         : <title>
-- Project       : <project>
-------------------------------------------------------------------------------
-- File          : testbench.vhd
-- Author        : 
-- Created       : 
-- Last modified : 1999/10/08
-------------------------------------------------------------------------------
-- Description :
-- <description>
-------------------------------------------------------------------------------
-- Modification history :
-- <modhist>
-------------------------------------------------------------------------------
LIBRARY IEEE;
  USE IEEE.STD_LOGIC_1164.ALL;

entity TESTBENCH is

    generic (
             test1 : boolean := false;
             test2 : boolean := false;
             test3 : boolean := false;
             test4 : boolean := false;
             limit : integer := 2;       -- rd/wr iteration limit

             CLOCK_PERIOD       : time := 30 ns;    -- Clock cycle period
             VCLK2X_HALF_PERIOD : time := 18 ns;    -- VCLK2X clock 1/2 period
             RESET_TIME         : time := 200 ns);  -- Reset low period
             
    

end TESTBENCH;

architecture behavior of TESTBENCH is

    ---------------------------------------------------------------------------
    -- COMPONENT DECLARATIONS
    ---------------------------------------------------------------------------
    
    
    COMPONENT amcc5933
        PORT
            (
                WR:            in       STD_LOGIC;
                RD:            in       STD_LOGIC;
                CS:            in       STD_LOGIC;
                MODE:          in       STD_LOGIC;
                WRFIFO:        in       STD_LOGIC;
                RDFIFO:        in       STD_LOGIC;
                BE:            in       STD_LOGIC_VECTOR(3 downto 0);
                ADR:           in       STD_LOGIC_VECTOR(4 downto 0);
                DQ:         inout       STD_LOGIC_VECTOR(31 downto 0);
                WRFULL:        out      STD_LOGIC;
                RDEMPTY:       out      STD_LOGIC;
                pt_cyc:        IN       std_logic;  
                pcontrol:      in       STD_LOGIC_VECTOR(7 downto 0);
                paddr :        in       STD_LOGIC_VECTOR(31 downto 0);
                pdata :        in       STD_LOGIC_VECTOR(31 downto 0);
                fill_pci_fifo: in       STD_LOGIC := '0';
                PTRDY:         IN       std_logic;  
                PTADR:         in       STD_LOGIC;
                PTATN:        out       STD_LOGIC;
                PTBURST:      out       STD_LOGIC;
                PTWR:         out       STD_LOGIC;
                PTNUM:        out       STD_LOGIC_VECTOR(1 downto 0);
                PTBE:         out       STD_LOGIC_VECTOR(3 downto 0);
                SYSRST:       BUFFER    STD_LOGIC;
                BPCLK:        BUFFER    STD_LOGIC;
                IRQ:          out       STD_LOGIC;
                CLK:          in        STD_LOGIC;
                RST:          in        STD_LOGIC;
                INTA:        out        STD_LOGIC

           );

    END COMPONENT;      

    ---------------------------------------------------------------------------
    -- SIGNAL DECLARATIONS
    ---------------------------------------------------------------------------
    
    signal N220: std_logic;
    
    for ALL: amcc5933 
        use entity board_comps.amcc5933(behavior);
---------------------------------------------------------------------------
-- BEGIN TEST CODE
---------------------------------------------------------------------------
    
begin

pullup <= '1';pulldown <= '0';



tests_proc : PROCESS

    variable k : integer := 0;
BEGIN  -- PROCESS tests_proc

   
    PCI_RST_L <= '0';
    T_PT_CYCLE <= '0';

    WAIT FOR RESET_TIME ;

    PCI_RST_L <= '1';

    WAIT FOR 8 * CLOCK_PERIOD ;

    
    ---------------------------------------------------------------------------
    -- TEST 1 Local Bus reads. 
    ---------------------------------------------------------------------------
     
    IF test1 THEN

          ---------------------------------------------------------------------
          -- T_PCONTROL = pbe 3 - 0 , pnum 1 - 0, pburst, pw
          ---------------------------------------------------------------------
          
          T_pcontrol  <= "11110011" ;
          T_paddr  <= x"11111111";
          I_DATA  <= x"55555555";

         for i in 0 to limit loop

            T_PT_CYCLE <= '1';
            wait for 2 * CLOCK_PERIOD;
            T_PT_CYCLE <= '0';

            wait until ptrdy_l = '0';
              wait for 2 * CLOCK_PERIOD;
            
         end loop;  -- i
        
    END IF;


    ---------------------------------------------------------------------------
    -- TEST 2 Local Bus writes. 
    ---------------------------------------------------------------------------
     
    IF test2 THEN

          ---------------------------------------------------------------------
          -- T_PCONTROL = pbe 3 - 0 , pnum 1 - 0, pburst, pw
          ---------------------------------------------------------------------
          
          T_pcontrol  <= "11110010" ;
          T_paddr  <= x"22222222";
          I_DATA  <= x"55555555";

         for i in 0 to limit loop

            T_PT_CYCLE <= '1';
            wait for 2 * CLOCK_PERIOD;
            T_PT_CYCLE <= '0';
            wait until ptrdy_l = '0';
              wait for 2 * CLOCK_PERIOD;
            
         end loop;  -- i
        
    END IF;


    ---------------------------------------------------------------------------
    -- TEST 3 Local Bus burst writes. 
    ---------------------------------------------------------------------------
     
    IF test3 THEN

          ---------------------------------------------------------------------
          -- T_PCONTROL = pbe 3 - 0 , pnum 1 - 0, pburst, pw
          ---------------------------------------------------------------------
          
          T_pcontrol  <= "11110000" ;
          T_paddr  <= x"33333333";
          I_DATA  <= x"CCCCCCCC";

         for i in 0 to 3 loop

            T_PT_CYCLE <= '1';
            wait for 2 * CLOCK_PERIOD;
            T_PT_CYCLE <= '0';

            for j in 0 to 3 loop
                wait until ptrdy_l = '0';
            end loop;  -- j
            
            wait for 2 * CLOCK_PERIOD;
            
         end loop;  -- i
        
    END IF;


    ---------------------------------------------------------------------------
    -- TEST 4 Local Bus burst reads. 
    ---------------------------------------------------------------------------
     
    IF test4 THEN

          ---------------------------------------------------------------------
          -- T_PCONTROL = pbe 3 - 0 , pnum 1 - 0, pburst, pw
          ---------------------------------------------------------------------
          
          T_pcontrol  <= "11110001" ;
          T_paddr  <= x"44444444";
          I_DATA  <= x"CCCCCCCC";

         for i in 0 to 3 loop

            T_PT_CYCLE <= '1';
            wait for 2 * CLOCK_PERIOD;
            T_PT_CYCLE <= '0';
            
            for j in 0 to 3 loop
                wait until ptrdy_l = '0';
            end loop;  -- j
            wait for 2 * CLOCK_PERIOD;
            
         end loop;  -- i
        
          ---------------------------------------------------------------------
          -- Now try bursting into the FIFO
          ---------------------------------------------------------------------

          RUN_FIFO <= '1';

            wait for 1600 * CLOCK_PERIOD;
    END IF;

    ---------------------------------------------------------------------------
    -- Done Testing - finish by doing nothing for 32 clocks.
    ---------------------------------------------------------------------------
    
   T_PT_CYCLE <= '0';
    --wait for 128 * CLOCK_PERIOD;
    wait for 2000000 * CLOCK_PERIOD;
    assert false report "################## TEST COMPLETE ##################" severity failure;
    
END PROCESS tests_proc;


T_pdata <= I_DATA when RUN_FIFO = '0' else (PATTERNS & PATTERNS);


end behavior;
